@@ -25,6 +25,7 @@ class Editor(SignalManager):
     request_transient_for = Signal(object)
     file_saved = Signal()
     push_escape_callback = Signal(object, object)
+    plugins_changed = Signal()
 
     def __init__(self):    
         self.uri = None
@@ -137,7 +138,9 @@ class Editor(SignalManager):
         return FeedbackPopup(self.view)
                 
     def message(self, message, timeout=1500):
-        self.feedback_popup.show(message, timeout)
+        popup = self.feedback_popup
+        popup.show(message, timeout)
+        self.push_escape(popup.hide)
 
     def push_escape(self, callback, *args):
         self.push_escape_callback.emit(callback, args)
@@ -273,21 +276,8 @@ class EditorManager(object):
                 idle(self.set_editor_prefs, e, e.uri)
         
     def new_file_action(self, editor):
-        dialog = gtk.FileChooserDialog('Create file', None, gtk.FILE_CHOOSER_ACTION_SAVE,
-            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
-        
-        root = editor.project_root
-        if root:
-            dialog.set_current_folder(root)
-
-        dialog.set_do_overwrite_confirmation(True)
-        self.set_transient_for(editor, dialog)
-        
-        response = dialog.run()
-        if response == gtk.RESPONSE_OK:
-            self.open(dialog.get_filename())
-            
-        dialog.destroy()
+        from snaked.core.gui import new_file
+        new_file.show_create_file(editor)
         
     def quit(self, *args):
         if self.session:
@@ -341,3 +331,8 @@ class EditorManager(object):
         from snaked.core.gui.editor_prefs import PreferencesDialog
         dialog = PreferencesDialog(self.lang_prefs)
         dialog.show(editor)
+
+    @Editor.plugins_changed
+    def on_plugins_changed(self, editor):
+        for e in self.editors:
+            self.set_editor_shortcuts(e)
\ No newline at end of file
